@@ -34,6 +34,8 @@ See module ``ebnf`` for a sample of the implementation of a
 compiler object.
 """
 
+from __future__ import annotations
+
 from collections import namedtuple
 import copy
 import functools
@@ -43,14 +45,14 @@ from typing import Any, Optional, Tuple, List, Set, Dict, Union, Callable, cast
 
 from DHParser.configuration import get_config_value
 from DHParser.preprocess import PreprocessorFunc
-from DHParser.nodetree import Node, RootNode, EMPTY_PTYPE, Trail
+from DHParser.nodetree import Node, RootNode, EMPTY_PTYPE, Path
 from DHParser.transform import TransformerCallable
 from DHParser.parse import Grammar
 from DHParser.preprocess import gen_neutral_srcmap_func
 from DHParser.error import is_error, is_fatal, Error, FATAL, \
     TREE_PROCESSING_CRASH, COMPILER_CRASH, AST_TRANSFORM_CRASH, has_errors
 from DHParser.log import log_parsing_history, log_ST, is_logging
-from DHParser.toolkit import load_if_file, is_filename, re
+from DHParser.toolkit import load_if_file, is_filename, re, TypeAlias
 
 
 __all__ = ('CompilerError',
@@ -59,10 +61,9 @@ __all__ = ('CompilerError',
            'CompilerFactory',
            'CompilationResult',
            'process_tree',
+           'Junction',
            'run_pipeline',
-           'compile_source',
-           'visitor_name',
-           'attr_visitor_name')
+           'compile_source')
 
 
 class CompilerError(Exception):
@@ -75,39 +76,8 @@ class CompilerError(Exception):
     pass
 
 
-def visitor_name(node_name: str) -> str:
-    """
-    Returns the visitor_method name for `name`, e.g.::
-
-        >>> visitor_name('expression')
-        'on_expression'
-        >>> visitor_name('!--')
-        'on_212d2d'
-    """
-    if re.match(r'\w+$', node_name):
-        return 'on_' + node_name
-    else:
-        letters = []
-        for ch in node_name:
-            if not re.match(r'\w', ch):
-                ch = hex(ord(ch))[2:]
-            letters.append(ch)
-        return 'on_' + ''.join(letters)
-
-
-def attr_visitor_name(attr_name: str) -> str:
-    """
-    Returns the visitor_method name for `attr_name`, e.g.::
-
-        >>> attr_visitor_name('class')
-        'attr_class'
-    """
-    return 'attr_' + attr_name
-
-
-
 ROOTNODE_PLACEHOLDER = RootNode()
-CompilerFunc = Callable[[Node], Any]
+CompilerFunc: TypeAlias = Callable[[Node], Any]
 
 
 class Compiler:
@@ -117,7 +87,7 @@ class Compiler:
     syntax tree (AST) as argument and return the compiled code in a
     format chosen by the compiler itself.
 
-    Subclasses implementing a compiler must define `on_XXX()`-methods
+    Subclasses implementing a compiler must define ``on_XXX()``-methods
     for each node name that can occur in the AST where 'XXX' is the
     node's name(for unnamed nodes it is the node's ptype without the
     leading colon ':').
@@ -126,54 +96,57 @@ class Compiler:
     argument. Other than in the AST transformation, which runs depth-first,
     compiler methods are called forward moving starting with the root
     node, and they are responsible for compiling the child nodes
-    themselves. This should be done by invoking the `compile(node)`-
-    method which will pick the right `on_XXX`-method. It is not
-    recommended to call the `on_XXX`-methods directly.
+    themselves. This should be done by invoking the ``compile(node)``-
+    method which will pick the right ``on_XXX()``-method or,
+    more commonly, by calling ``fallback_compiler()``-methods which
+    compiles of child-nodes and updates the tuple of children according
+    to the results. It is not recommended to call the ``on_XXX()``-methods
+    directly!
 
     Variables that are (re-)set only in the constructor and retain their
-    value if changed during subesquent calls::
+    value if changed during subesquent calls:
 
     :ivar forbid_returning_None:  Default value: True. Most of the time,
-            if a compiler-method (i.e. on_XXX()) returns None, this is
-            a mistake due to a forgotten return statement. The method
-            compile() checks for this mistake and raises an error if
-            a compiler-method returns None. However, some compilers require
-            the possibility to return None-values. In this case
-            ``forbis_returing_None`` should be set to False in the constructor
-            of the derived class.
+        if a compiler-method (i.e. on_XXX()) returns None, this is
+        a mistake due to a forgotten return statement. The method
+        compile() checks for this mistake and raises an error if
+        a compiler-method returns None. However, some compilers require
+        the possibility to return None-values. In this case
+        ``forbis_returing_None`` should be set to False in the constructor
+        of the derived class.
 
-            (Another Alternativ would be to return a sentinel object instead of
-            None.)
+        (Another Alternativ would be to return a sentinel object instead of
+        None.)
 
-    Object-Variables that are reset after each call of the Compiler-object::
+    Object-Variables that are reset after each call of the Compiler-object:
 
-    :ivar trail:  A list of parent nodes that ends with the currently
-                compiled node.
+    :ivar path:  A list of parent nodes that ends with the currently
+        compiled node.
     :ivar tree: The root of the abstract syntax tree.
     :ivar finalizers:  A stack of tuples (function, parameters) that will be
-                called in reverse order after compilation.
+        called in reverse order after compilation.
 
     :ivar has_attribute_visitors:  A flag indicating that the class has
-                attribute-visitor-methods which are named 'attr_ATTRIBUTENAME'
-                and will be called if the currently processed node has one
-                or more attributes for which such visitors exist.
+        attribute-visitor-methods which are named 'attr_ATTRIBUTENAME'
+        and will be called if the currently processed node has one
+        or more attributes for which such visitors exist.
 
     :ivar _dirty_flag:  A flag indicating that the compiler has already been
-                called at least once and that therefore all compilation
-                variables must be reset when it is called again.
+        called at least once and that therefore all compilation
+        variables must be reset when it is called again.
     :ivar _debug: A flag indicating that debugging is turned on. The value
-                for this flag is read before each call of the configuration
-                (see debugging section in DHParser.configuration).
-                If debugging is turned on the compiler class raises en
-                error if there is an attempt to be compile one and the same
-                node a second time..
+        for this flag is read before each call of the configuration
+        (see debugging section in :py:mod:`DHParser.configuration`).
+        If debugging is turned on the compiler class raises en
+        error if there is an attempt to be compiled one and the same
+        node a second time.
     :ivar _debug_already_compiled: A set of nodes that have already been compiled.
     """
 
     def __init__(self):
-        self.has_attribute_visitors = any(field[0:5] == 'attr_' and callable(getattr(self, field))
+        self.has_attribute_visitors: bool = any(field[0:5] == 'attr_' and callable(getattr(self, field))
                                           for field in dir(self))
-        self.forbid_returning_None = True  # type: bool
+        self.forbid_returning_None: bool = True
         self.reset()
 
     def reset(self):
@@ -182,14 +155,44 @@ class Compiler:
         of the object.
         """
         self.tree = ROOTNODE_PLACEHOLDER   # type: RootNode
-        self.trail = []  # type: Trail
+        self.path = []  # type: Path
         self._dirty_flag = False
         self._debug = get_config_value('debug_compiler')  # type: bool
         self._debug_already_compiled = set()              # type: Set[Node]
         self.finalizers = []  # type: List[Tuple[Callable, Tuple]]
         self.method_dict = {}  # type: Dict[str, CompilerFunc]
 
-    def prepare(self, root: Node) -> None:
+    def visitor_name(self, node_name: str) -> str:
+        """
+        Returns the visitor_method name for `name`, e.g.::
+
+            >>> c = Compiler()
+            >>> c.visitor_name('expression')
+            'on_expression'
+            >>> c.visitor_name('!--')
+            'on_212d2d'
+        """
+        if re.match(r'\w+$', node_name):
+            return 'on_' + node_name
+        else:
+            letters = []
+            for ch in node_name:
+                if not re.match(r'\w', ch):
+                    ch = hex(ord(ch))[2:]
+                letters.append(ch)
+            return 'on_' + ''.join(letters)
+
+    def attr_visitor_name(self, attr_name: str) -> str:
+        """
+        Returns the visitor_method name for `attr_name`, e.g.::
+
+            >>> c = Compiler()
+            >>> c.attr_visitor_name('class')
+            'attr_class'
+        """
+        return 'attr_' + attr_name.replace('-', '_').replace('.', '_').replace(':', '_')
+
+    def prepare(self, root: RootNode) -> None:
         """
         A preparation method that will be called after everything else has
         been initialized and immediately before compilation starts. This method
@@ -204,6 +207,18 @@ class Compiler:
         """
         return result
 
+    def wildcard(self, node: Node) -> Any:
+        """
+        The wildcard method is called nodes for which no other compilation-
+        method has been specified. This allows to check, whether illegal
+        nodes occur in the tree (although, a static structural validation
+        is to be preferred.) or whether a compilation node has been
+        forgotten.
+
+        Per default, wildcard() just redirects to self.fallback_compiler()
+        """
+        return self.fallback_compiler(node)
+
     def __call__(self, root: Node) -> Any:
         """
         Compiles the abstract syntax tree with the root node `node` and
@@ -240,17 +255,16 @@ class Compiler:
         result = self.finalize(result)
         return result
 
-    def visit_attributes(self, node) -> Node:
+    def visit_attributes(self, node):
         if node.has_attr():
-            self.trail.append(node)
-            for attribute, value in list(node.attr.items()):
+            self.path.append(node)
+            for attribute, value in tuple(node.attr.items()):
                 try:
-                    attribute_visitor = self.__getattribute__(attr_visitor_name(attribute))
-                    node = attribute_visitor(node, value) or node
+                    attribute_visitor = self.__getattribute__(self.attr_visitor_name(attribute))
+                    attribute_visitor(node, value) or node
                 except AttributeError:
                     pass
-            self.trail.pop()
-        return node
+            self.path.pop()
 
     def fallback_compiler(self, node: Node) -> Any:
         """This is a generic compiler function which will be called on
@@ -280,18 +294,21 @@ class Compiler:
                         result.append(nd)
                 node.result = tuple(result)
         if self.has_attribute_visitors:
-            node = self.visit_attributes(node)
+            self.visit_attributes(node)
         return node
 
     def find_compilation_method(self, node_name: str) -> CompilerFunc:
         try:
             method = self.method_dict[node_name]
         except KeyError:
-            method_name = visitor_name(node_name)
+            method_name = self.visitor_name(node_name)
             try:
                 method = self.__getattribute__(method_name)
             except AttributeError:
-                method = self.fallback_compiler
+                if self.wildcard != Compiler.wildcard:
+                    method = self.wildcard
+                else:
+                    method = self.fallback_compiler
             self.method_dict[node_name] = method
         return method
 
@@ -316,9 +333,9 @@ class Compiler:
         if elem[:1] == ':':
             elem = elem[1:] + '__'
         compiler = self.find_compilation_method(elem)
-        self.trail.append(node)
+        self.path.append(node)
         result = compiler(node)     
-        self.trail.pop()
+        self.path.pop()
         if self.has_attribute_visitors:
             node = self.visit_attributes(node)          
         if result is None and self.forbid_returning_None:
@@ -350,8 +367,8 @@ def logfile_basename(filename_or_text, function_or_class_or_instance) -> str:
 # processing pipeline support #########################################
 
 
-CompilerCallable = Union[Compiler, Callable[[RootNode], Any], functools.partial]
-CompilerFactory = Union[Callable[[], CompilerCallable], functools.partial]
+CompilerCallable: TypeAlias = Union[Compiler, Callable[[RootNode], Any], functools.partial]
+CompilerFactory: TypeAlias = Union[Callable[[], CompilerCallable], functools.partial]
 
 
 def filter_stacktrace(stacktrace: List[str]) -> List[str]:
@@ -395,8 +412,8 @@ def process_tree(tp: CompilerCallable, tree: RootNode) -> Any:
             try:
                 result = tp(tree)
             except Exception as e:
-                # node = tp.trail[-1] if hasattr(tp, 'trail') and tp.trail else tree
-                node = getattr(tp, 'trail', [tree])[-1]
+                # node = tp.path[-1] if hasattr(tp, 'path') and tp.path else tree
+                node = getattr(tp, 'path', [tree])[-1]
                 st = traceback.format_list(traceback.extract_tb(e.__traceback__))
                 trace = ''.join(st)  # filter_stacktrace(st)
                 tree.new_error(
@@ -411,7 +428,7 @@ def process_tree(tp: CompilerCallable, tree: RootNode) -> Any:
     return result
 
 
-Junction = Tuple[str, CompilerFactory, str]  # source stage name, source->target, target stage name
+Junction: TypeAlias = Tuple[str, CompilerFactory, str]  # source stage name, source->target, target stage name
 
 
 def run_pipeline(junctions: Set[Junction],
@@ -430,13 +447,22 @@ def run_pipeline(junctions: Set[Junction],
 
     TODO: Parallelize processing of junctions
     """
+    def cmp_junctions(a, b) -> int:
+        if a[-1] == b[0]:
+            return -11
+        if b[-1] == a[0]:
+            return 1
+        return 0
+
     t_to_j = {j[-1]: j for j in junctions}
     steps = []
     targets = target_stages.copy()
     already_reached = targets.copy() | source_stages.keys()
     while targets:
         try:
-            steps.append([t_to_j[t] for t in targets if t not in source_stages])
+            j_sequence = [t_to_j[t] for t in targets if t not in source_stages]
+            j_sequence.sort(key=functools.cmp_to_key(cmp_junctions))
+            steps.append(j_sequence)
         except KeyError as e:
             raise AssertionError(f"{e.args[0]} is not a valid target.")
         targets = {j[0] for j in steps[-1] if j[0] not in already_reached}
@@ -474,7 +500,7 @@ def run_pipeline(junctions: Set[Junction],
                                          f'and, therefore, cannot be processed to {t}')
                     results[t] = process_tree(junction[1](), tree)
                     errata[t] = copy.copy(tree.errors_sorted)
-    return {t: (results[t], errata[t]) for t in target_stages}
+    return {t: (results[t], errata[t]) for t in results.keys()}
 
 
 CompilationResult = namedtuple('CompilationResult',
@@ -549,7 +575,8 @@ def compile_source(source: str,
 
     # parsing
 
-    syntax_tree = parser(source_text, source_mapping=source_mapping)  # type: RootNode
+    syntax_tree: RootNode = parser(source_text, source_mapping=source_mapping)
+    syntax_tree.docname = source_name if source_name else "DHParser_Document"
     for e in errors:  syntax_tree.add_error(None, e)
     syntax_tree.source = original_text
     syntax_tree.source_mapping = source_mapping
