@@ -47,13 +47,15 @@ column-number
 
 """
 
+from __future__ import annotations
+
 from collections import namedtuple
 import functools
 import os
 from typing import Iterable, Iterator, Union, List, Sequence, Dict, Callable
 
 from DHParser.stringview import StringView
-from DHParser.toolkit import linebreaks, line_col, is_filename
+from DHParser.toolkit import linebreaks, line_col, is_filename, TypeAlias
 
 
 __all__ = ('SourceMap',
@@ -169,8 +171,7 @@ SourceLocation = namedtuple('SourceLocation',
      'pos',  # type: int
     ], module=__name__)
 
-SourceMapFunc = Union[Callable[[int], SourceLocation],
-                      functools.partial]
+SourceMapFunc: TypeAlias = Union[Callable[[int], SourceLocation], functools.partial]
 
 
 #######################################################################
@@ -346,8 +347,10 @@ class Error:
             prefix = self.orig_doc + ':'
         else:  prefix = ''
         if self.line > 0:
-            prefix += "%i:%i: " % (max(self.line, 0), max(self.column, 0))
-        return prefix + "%s (%i): %s" % (self.severity, self.code, self.message)
+            # prefix += "%i:%i: " % (max(self.line, 0), max(self.column, 0))
+            prefix += f"{max(self.line, 0)}:{max(self.column, 0)}: "
+        # return prefix + "%s (%i): %s" % (self.severity, self.code, self.message)
+        return prefix + f"{self.severity} ({self.code}): {self.message}"
 
     def __repr__(self):
         return 'Error("%s", %s, %i, %i, %i, %i)' \
@@ -519,7 +522,7 @@ def canonical_error_strings(errors: List[Error]) -> List[str]:
             if source_file_name and is_filename(source_file_name):
                 cwd = os.getcwd()
                 if source_file_name.startswith(cwd):
-                    rel_path = source_file_name[len(cwd)]
+                    rel_path = source_file_name[len(cwd):]
                 else:
                     rel_path = source_file_name
                 err_str = str(err)
