@@ -23,6 +23,7 @@ implied. See the License for the specific language governing
 permissions and limitations under the License.
 """
 
+# from __future__ import annotations  # commented out, because it crashes :-(
 
 from enum import Enum, IntEnum
 import functools
@@ -43,7 +44,7 @@ except ImportError:
     _GenericAlias = GenericMeta
     _SpecialForm = Any
 try:
-   from typing_extensions import get_origin
+   from DHParser.externallibs.typing_extensions import get_origin
 except ImportError:
     def get_origin(typ):
         try:
@@ -138,7 +139,26 @@ def _new_typed_dict(meta, name, bases, ns) -> dict:
 
     total = True
     for field, field_type in own_annotations.items():
-        if get_origin(field_type) is Union \
+        if isinstance(field_type, ForwardRef):
+            field_type_str = field_type.__forward_arg__
+            if field_type_str[:5] == 'Union':
+                i = 5
+                while i >= 0:
+                    i = field_type_str.find('None', i + 1)
+                    if i >= 0:
+                        rest = field_type_str[i + 5:]
+                        if rest.count('[') >= rest.count(']'):
+                            optional_keys.add(field)
+                            total = False
+                            i = -1
+                    else:
+                        required_keys.add(field)
+            elif field_type_str[:8] == 'Optional':
+                optional_keys.add(field)
+                total = False
+            else:
+                required_keys.add(field)
+        elif get_origin(field_type) is Union \
                 and type(None) in field_type.__args__:
             optional_keys.add(field)
             total = False
@@ -289,7 +309,7 @@ def validate_type(val: Any, typ):
     ...     validate_type(bad_json_data, Position)
     ... except TypeError as e:
     ...     print(e)
-    Type error(s) in dictionary of type <class 'json_validation.Position'>:
+    Type error(s) in dictionary of type <class 'DHParser.json_validation.Position'>:
     Field character: 'A' is not a <class 'int'>, but a <class 'str'>
     """
     if isinstance(typ, _TypedDictMeta):
@@ -395,7 +415,7 @@ def validate_TypedDict(D: Dict, T: _TypedDictMeta):
     ...     validate_TypedDict(p, Position)
     ... except TypeError as e:
     ...     print(e)
-    Type error(s) in dictionary of type <class 'json_validation.Position'>:
+    Type error(s) in dictionary of type <class 'DHParser.json_validation.Position'>:
     Missing required keys: {'character'}
 
     :param D: the dictionary to be validated
@@ -487,9 +507,9 @@ def type_check(func: Callable, check_return_type: bool = True) -> Callable:
     ... except TypeError as e:
     ...     print(e)
     Parameter "rng" of function "middle_line" failed the type-check, because:
-    Type error(s) in dictionary of type <class 'json_validation.Range'>:
-    Field start: '1' is not of <class 'json_validation.Position'>, but of type <class 'int'>
-    Field end: '8' is not of <class 'json_validation.Position'>, but of type <class 'int'>
+    Type error(s) in dictionary of type <class 'DHParser.json_validation.Range'>:
+    Field start: '1' is not of <class 'DHParser.json_validation.Position'>, but of type <class 'int'>
+    Field end: '8' is not of <class 'DHParser.json_validation.Position'>, but of type <class 'int'>
 
     :param func: The function, the parameters and return value of which shall
         be type-checked during runtime.
