@@ -51,18 +51,25 @@ Example::
                                          transformer, compiler)
 """
 
+from __future__ import annotations
+
 import collections
 import contextlib
 import html
 import os
 from typing import List, Tuple, Union, Optional, Sequence
 
+try:
+    import cython
+except ImportError:
+    import DHParser.externallibs.shadow_cython as cython
+
 from DHParser.configuration import access_presets, finalize_presets, get_config_value, \
     set_config_value, get_preset_value, set_preset_value
 from DHParser.error import Error
 from DHParser.stringview import StringView
 from DHParser.nodetree import Node, FrozenNode, ZOMBIE_TAG, EMPTY_PTYPE
-from DHParser.toolkit import escape_ctrl_chars, abbreviate_middle, cython
+from DHParser.toolkit import escape_ctrl_chars, abbreviate_middle, TypeAlias
 
 __all__ = ('CallItem',
            'start_logging',
@@ -82,7 +89,7 @@ __all__ = ('CallItem',
            'log_parsing_history')
 
 
-CallItem = Tuple[str, int]      # call stack item: (name, location)
+CallItem: TypeAlias = Tuple[str, int]      # call stack item: (name, location)
 
 
 #######################################################################
