@@ -27,6 +27,7 @@ for some domain specific languages, most notably those that
 cannot completely be described entirely with context-free grammars.
 """
 
+from __future__ import annotations
 
 import bisect
 from collections import namedtuple
@@ -37,7 +38,7 @@ from typing import Union, Optional, Callable, Tuple, List, Any
 from DHParser.error import Error, SourceMap, SourceLocation, SourceMapFunc, \
     add_source_locations
 from DHParser.stringview import StringView
-from DHParser.toolkit import re
+from DHParser.toolkit import re, TypeAlias
 
 
 __all__ = ('RX_TOKEN_NAME',
@@ -109,11 +110,13 @@ PreprocessorResult = namedtuple('PreprocessorResult',
     module=__name__)
 
 
-FindIncludeFunc = Union[Callable[[str, int], IncludeInfo],   # (document: str,  start: int)
-                        functools.partial]
-PreprocessorFunc = Union[Callable[[str, str], PreprocessorResult],  # text: str, filename: str
-                         functools.partial]
-Tokenizer = Union[Callable[[str], Tuple[str, List[Error]]], functools.partial]
+FindIncludeFunc: TypeAlias = Union[Callable[[str, int], IncludeInfo],   # (document: str,  start: int)
+                                   functools.partial]
+DeriveFileNameFunc: TypeAlias = Union[Callable[[str], str], functools.partial]  # include name -> file name
+PreprocessorFunc: TypeAlias = Union[Callable[[str, str], PreprocessorResult],  # text: str, filename: str
+                                    functools.partial]
+Tokenizer: TypeAlias = Union[Callable[[str], Tuple[str, List[Error]]],
+                             functools.partial]
 
 # a functions that merely adds preprocessor tokens to a source text
 
@@ -336,7 +339,15 @@ def make_preprocessor(tokenizer: Tokenizer) -> PreprocessorFunc:
 
 
 def gen_find_include_func(rx: Union[str, Any],
-                          comment_rx: Optional[Union[str, Any]] = None) -> FindIncludeFunc:
+                          comment_rx: Optional[Union[str, Any]] = None,
+                          derive_file_name: DeriveFileNameFunc = lambda name: name) \
+                          -> FindIncludeFunc:
+    """Generates a function to find include-statements in a file.
+
+    :param rx: A regular expression (either as string or compiled
+        regular expression) to catch the names of the includes in
+        a document. The expression should catch
+    """
     if isinstance(rx, str):  rx = re.compile(rx)
     if isinstance(comment_rx, str):  comment_rx = re.compile(comment_rx)
 
@@ -345,7 +356,8 @@ def gen_find_include_func(rx: Union[str, Any],
         m = rx.search(text, begin)
         if m:
             begin = m.start()
-            return IncludeInfo(begin, m.end() - begin, m.group('name'))
+            file_name = derive_file_name(m.group('name'))
+            return IncludeInfo(begin, m.end() - begin, file_name)
         else:
             return IncludeInfo(-1, 0, '')
 
