@@ -25,6 +25,7 @@ Usually, however, unexpected failure to match a certain string is the
 main cause of trouble when constructing a context free Grammar.
 """
 
+from __future__ import annotations
 
 import asyncio
 import collections
@@ -46,7 +47,7 @@ if sys.version_info >= (3, 6, 0):
 else:
     from collections import OrderedDict
 
-from DHParser.configuration import get_config_value
+from DHParser.configuration import get_config_value, set_config_value
 from DHParser.compile import run_pipeline
 from DHParser.error import Error, is_error, PARSER_LOOKAHEAD_MATCH_ONLY, \
     PARSER_LOOKAHEAD_FAILURE_ONLY, MANDATORY_CONTINUATION_AT_EOF, \
@@ -55,10 +56,10 @@ from DHParser.error import Error, is_error, PARSER_LOOKAHEAD_MATCH_ONLY, \
 from DHParser.log import is_logging, clear_logs, local_log_dir, log_parsing_history
 from DHParser.parse import Lookahead
 from DHParser.server import RX_CONTENT_LENGTH, RE_DATA_START, JSONRPC_HEADER_BYTES
-from DHParser.nodetree import Node, RootNode, deserialize, flatten_sxpr, ZOMBIE_TAG, EMPTY_PTYPE
+from DHParser.nodetree import Node, RootNode, deserialize, flatten_sxpr, ZOMBIE_TAG
 from DHParser.trace import set_tracer, all_descendants, trace_history
 from DHParser.transform import traverse, remove_children
-from DHParser.toolkit import load_if_file, re, re_find, concurrent_ident, instantiate_executor
+from DHParser.toolkit import load_if_file, re, re_find, instantiate_executor, TypeAlias
 
 
 __all__ = ('unit_from_config',
@@ -279,6 +280,8 @@ def get_report(test_unit) -> str:
         return "\n    ".join(lines)
 
     report = []
+    save = get_config_value('xml_attribute_error_handling')
+    set_config_value('xml_attribute_error_handling', 'fix')
     for parser_name, tests in test_unit.items():
         heading = 'Test of parser: "%s"' % parser_name
         report.append('\n\n%s\n%s' % (heading, '=' * len(heading)))
@@ -308,7 +311,7 @@ def get_report(test_unit) -> str:
                     result = tests[stage][test_name]
                     report.append(f'\n### {stage.strip("_")}\n')
                     if isinstance(result, Node):
-                        result_str = cast(Node, result).serialize('ast')
+                        result_str = cast(Node, result).serialize('default')
                     else:
                         result_str = str(result)
                     report.append(indent(result_str))
@@ -326,6 +329,7 @@ def get_report(test_unit) -> str:
             if error:
                 report.append('\n### Error:\n')
                 report.append(error)
+    set_config_value('xml_attribute_error_handling', save)
     return '\n'.join(report)
 
 
@@ -465,7 +469,6 @@ def grammar_unit(test_unit, parser_factory, transformer_factory, report='REPORT'
         match_tests = set(tests['match'].keys()) if 'match' in tests else set()
         match_test_keys = {clean_key(k) for k in match_tests}
 
-
         transformation_stages = {key for key in tests if key not in {'match', 'fail'}}
         for stage in transformation_stages:
             transformation_tests = set(tests[stage].keys())
@@ -534,12 +537,21 @@ def grammar_unit(test_unit, parser_factory, transformer_factory, report='REPORT'
                     raise SyntaxError(f'Compilation-Test {test_name} of parser {parser_name} '
                                       f'failed with:\n{str(e)}')
                 t_errors: Dict[str, List[Error]] = {}
-                for stage in transformation_stages | show:
+                for stage in list(transformation_stages) + [t for t in targets if t in show]:
                     tests.setdefault(f'__{stage}__', {})[test_name] = targets[stage][0]
                     t_errors[stage] = [e for e in targets[stage][1] if e not in old_errors]
                     for e in t_errors[stage]:
                         old_errors.add(e)
                     add_errors_to_errata(t_errors[stage])
+                # keep test-items, so that the order of the items is the same as
+                # in which they are processed in the pipeline.
+                for t in targets:
+                    if t in show:
+                        k = f'__{t}__'
+                        if k in tests:
+                            save = tests[k]
+                            del tests[k]
+                            tests[k] = save
 
             if verbose:
                 infostr = '    match-test "' + test_name + '" ... '
@@ -782,7 +794,7 @@ def grammar_suite(directory, parser_factory, transformer_factory,
 
 
 RX_DEFINITION_OR_SECTION = re.compile(r'(?:^|\n)[ \t]*(\w+(?=[ \t]*=)|#:.*(?=\n|$|#))')
-SymbolsDictType = Dict[str, List[str]]
+SymbolsDictType: TypeAlias = Dict[str, List[str]]
 
 ALL_SYMBOLS = 'ALL_SYMBOLS'
 
