@@ -18,14 +18,17 @@
 
 """
 Module ``toolkit`` contains utility functions that are needed across
-several of the the other DHParser-Modules Helper functions that are not
+several of the other DHParser-Modules Helper functions that are not
 needed in more than one module are best placed within that module and
-not in the toolkit-module. An acceptable exception from this rule are
+not in the toolkit-module. An acceptable exception to this rule are
 functions that are very generic.
 """
 
+from __future__ import annotations
+
 import ast
 import bisect
+import collections.abc
 import concurrent.futures
 import functools
 import hashlib
@@ -57,6 +60,10 @@ try:
 except ImportError:
     class Protocol:
         pass
+try:
+    from typing import TypeAlias
+except ImportError:
+    from DHParser.externallibs.typing_extensions import TypeAlias
 
 try:
     import cython
@@ -67,7 +74,8 @@ except ImportError:
     cython_optimized = False
     import DHParser.externallibs.shadow_cython as cython
 
-from DHParser.configuration import access_thread_locals, get_config_value, NEVER_MATCH_PATTERN
+from DHParser.configuration import access_thread_locals, get_config_value, set_config_value, \
+    NEVER_MATCH_PATTERN
 from DHParser.stringview import StringView
 
 
@@ -75,11 +83,11 @@ __all__ = ('typing',
            're',
            'dataclasses',
            'Protocol',
-           'cython',
+           'TypeAlias',
            'cython_optimized',
            'identify_python',
            'identity',
-           'gen_id',
+           # 'gen_id',
            'ThreadLocalSingletonFactory',
            'RX_NEVER_MATCH',
            'RX_ENTITY',
@@ -98,6 +106,8 @@ __all__ = ('typing',
            'concurrent_ident',
            'unrepr',
            'abbreviate_middle',
+           'wrap_str_nicely',
+           'printw',
            'escape_formatstr',
            'as_identifier',
            'as_list',
@@ -152,17 +162,15 @@ def identity(x):
     return x
 
 
-if __name__ == "__main__":
-    global_id_counter = multiprocessing.Value('L', 0)
+
+global_id_counter: int = 0
 
 
-def gen_id() -> int:
-    """Generates a unique id."""
-    global global_id_counter
-    with global_id_counter.get_lock():
-        next_id = global_id_counter.value + 1
-        global_id_counter.value = next_id
-    return next_id
+# def gen_id() -> str:
+#     """Generates a unique id. (Not thread-safe!)"""
+#     global global_id_counter
+#     global_id_counter += 1
+#     return str(global_id_counter)
 
 
 class ThreadLocalSingletonFactory:
@@ -171,10 +179,12 @@ class ThreadLocalSingletonFactory:
     the same instance of `class_or_factory` for one and the
     same thread, but different instances for different threads.
     """
-    def __init__(self, class_or_factory, name: str = "", *, ident: int = -1):
+    def __init__(self, class_or_factory, name: str = "", *, ident: str = ""):
         self.class_or_factory = class_or_factory
-        self.singleton_name = "{NAME}_{ID:08d}_singleton".format(
-            NAME=name or class_or_factory.__name__, ID=ident if ident >= 0 else gen_id())
+        self.singleton_name = "{NAME}_{ID}_singleton".format(
+            NAME=name or class_or_factory.__name__, ID=str(id(self)))
+        THREAD_LOCALS = access_thread_locals()
+        assert not hasattr(THREAD_LOCALS, self.singleton_name), self.singleton_name
 
     def __call__(self):
         THREAD_LOCALS = access_thread_locals()
@@ -333,8 +343,13 @@ def deprecated(message: str) -> Callable:
         >>> @deprecated('This function is deprecated!')
         ... def bad():
         ...     pass
-        >>> bad()
+        >>> save = get_config_value('deprecation_policy')
+        >>> set_config_value('deprecation_policy', 'fail')
+        >>> try: bad()
+        ... except DeprecationWarning as w:  print(w)
         This function is deprecated!
+        >>> set_config_value('deprecation_policy', save)
+
     """
     assert isinstance(message, str)
     def decorator(f: Callable) -> Callable:
@@ -499,6 +514,92 @@ def abbreviate_middle(s: str, max_length: int) -> str:
     return s
 
 
+@cython.locals(wrap_column=cython.int, tolerance=cython.int, a=cython.int, i=cython.int, k=cython.int, m=cython.int)
+def wrap_str_nicely(s: str, wrap_column: int=79, tolerance: int=24,
+                    wrap_chars: str=")]>, ") -> str:
+    r"""Line-wraps a single-line output string at 'wrap_column'. Tries to
+    find a suitable point for wrapping, i.e. after any of the wrap_characters.
+
+    If the strings spans multiple lines, the existing linebreaks will be kept
+    and no rewrapping takes place. In order to enforce rewrapping of multiline
+    strings, use: ``wrap_str_nicely(repr(s)[1:-1])``. (repr() replaces
+    linebreaks by the \\n-marker. The slicing [1:-1] removes the opening
+    and closing angular quotation marks that repr adds.
+
+    Examples::
+
+    >>> s = ('(X (l ",.") (A (O "123") (P (:Text "4") (em "56"))) '
+    ...      '(em (m "!?")) (B (Q (em "78") (:Text "9")) (R "abc")) '
+    ...      '(n "+-"))')
+    >>> print(wrap_str_nicely(s))
+    (X (l ",.") (A (O "123") (P (:Text "4") (em "56"))) (em (m "!?"))
+     (B (Q (em "78") (:Text "9")) (R "abc")) (n "+-"))
+    >>> s = ('(X (s) (A (u) (C "One,")) (em (A (C " ") (D "two, ")))'
+    ...      '(B (E "three, ") (F "four!") (t))))')
+    >>> print(wrap_str_nicely(s))
+    (X (s) (A (u) (C "One,")) (em (A (C " ") (D "two, ")))(B (E "three, ")
+     (F "four!") (t))))
+    >>> s = ("[Node('word', 'This'), Node('word', 'is'), "
+    ...      "Node('word', 'Buckingham'), Node('word', 'Palace')]")
+    >>> print(wrap_str_nicely(s))
+    [Node('word', 'This'), Node('word', 'is'), Node('word', 'Buckingham'),
+     Node('word', 'Palace')]
+    >>> s = ("Node('phrase', (Node('word', 'Buckingham'), "
+    ...      "Node('blank', ' '), Node('word', 'Palace')))")
+    >>> print(wrap_str_nicely(s))
+    Node('phrase', (Node('word', 'Buckingham'), Node('blank', ' '),
+     Node('word', 'Palace')))
+    >>> s = ('<hard>Please mark up <foreign lang="de">Stadt\n<lb/></foreign>'
+    ...      '<location><foreign lang="de"><em>München</em></foreign> '
+    ...      'in Bavaria</location> in this sentence.</hard>')
+    >>> print(wrap_str_nicely(s))
+    <hard>Please mark up <foreign lang="de">Stadt
+    <lb/></foreign><location><foreign lang="de"><em>München</em></foreign> in Bavaria</location> in this sentence.</hard>
+    >>> print(wrap_str_nicely(repr(s)[1:-1]))  # repr to ignore the linebreaks
+    <hard>Please mark up <foreign lang="de">Stadt\n<lb/></foreign><location>
+    <foreign lang="de"><em>München</em></foreign> in Bavaria</location>
+     in this sentence.</hard>
+    """
+    assert 2 <= tolerance
+    assert wrap_column > tolerance
+    if len(s) <= wrap_column or s.rfind('\n') >= 0:  return s
+    parts = []
+    a = 0
+    i = wrap_column
+    while i < len(s):
+        for ch in wrap_chars:
+            m = i
+            while s[m] == ch:  m -= 1
+            if i - m > tolerance // 2:
+                continue
+            k = s.rfind(ch, a, m)
+            probe = s[k + 1]
+            while k < i and s[k + 1] in wrap_chars and s[k + 1] != ' ':  k += 1
+            if i - k <= tolerance:
+                parts.append(s[a:k + 1])
+                a = k + 1
+                i = k + 1 + wrap_column
+                break
+        else:
+            parts.append(s[a:i])
+            a = i
+            i += wrap_column
+    if a < len(s) - 1:  parts.append(s[a:])
+    return '\n'.join(parts)
+
+
+def printw(s: Any, wrap_column: int=79, tolerance: int=24,
+           wrap_chars: str=")]>, "):
+    """Prints the string or other object nicely wrapped.
+    See :py:func:`wrap_str_nicely`."""
+    if isinstance(s, (list, tuple, dict)) and wrap_chars == ")]>, ":
+        wrap_chars = ",)]> "
+        s = repr(s)
+    elif not isinstance(s, str):
+        s = repr(s)
+    print(wrap_str_nicely(s, wrap_column, tolerance, wrap_chars))
+
+
 def escape_formatstr(s: str) -> str:
     """Replaces single curly braces by double curly-braces in string `s`,
     so that they are not misinterpreted as place-holder by "".format().
@@ -673,28 +774,32 @@ def issubtype(sub_type, base_type) -> bool:
     with new Python-versions.
     """
     def origin(t) -> tuple:
+        def fix(t: Any) -> Any:
+            return {'Dict': dict, 'Tuple': tuple, 'List': list}.get(t, t)
         try:
+            if isinstance(t, (str, bytes)):  t = eval(t)
             ot = t.__origin__
             if ot is Union:
                 try:
-                    return tuple((a.__origin__ if a.__origin__ is not None else a)
+                    return tuple((fix(a.__origin__) if a.__origin__ is not None else fix(a))
                                  for a in t.__args__)
                 except AttributeError:
                     try:
-                        return tuple((a.__origin__ if a.__origin__ is not None else a)
+                        return tuple((fix(a.__origin__) if a.__origin__ is not None else fix(a))
                                      for a in t.__union_args__)
                     except AttributeError:
                         return t.__args__
         except AttributeError:
             if t == 'unicode':  # work-around for cython bug
                 return str,
-            return t,
-        return (ot,) if ot is not None else (t,)
+            return fix(t),
+        return (fix(ot),) if ot is not None else (fix(t),)
     true_st = origin(sub_type)
     true_bt = origin(base_type)[0]
     return any(issubclass(st, true_bt) for st in true_st)
 
 
+
 def isgenerictype(t):
     """Returns `True` if `t` is a generic type.
     WARNING: This is very "hackish". Caller must make sure that `t`
@@ -964,8 +1069,8 @@ def expand_table(compact_table: Dict) -> Dict:
 #######################################################################
 
 
-JSON_Type = Union[Dict, Sequence, str, int, float, None]
-JSON_Dict = Dict[str, JSON_Type]
+JSON_Type: TypeAlias = Union[Dict, Sequence, str, int, float, None]
+JSON_Dict: TypeAlias = Dict[str, JSON_Type]
 
 
 class JSONstr:
