@@ -350,7 +350,7 @@ def main():
 
     if os.path.exists(grammar_path) and os.path.isfile(grammar_path):
         if not recompile_grammar(grammar_path, script_path, force=False, notify=notify):
-            error_file = os.path.basename(__file__).replace('Parser.py', '_ebnf_ERRORS.txt')
+            error_file = os.path.basename(__file__).replace('Parser.py', '_ebnf_MESSAGES.txt')
             with open(error_file, 'r', encoding="utf-8") as f:
                 print(f.read())
             sys.exit(1)
