@@ -72,7 +72,7 @@ class FixedEBNFGrammar(Grammar):
     countable = Forward()
     element = Forward()
     expression = Forward()
-    source_hash__ = "dcfc8fa514a500d3e60a42b4eb1e9b49"
+    source_hash__ = "2874a4293e93b266ddf2612b70148bcf"
     disposable__ = re.compile('component$|pure_elem$|countable$|FOLLOW_UP$|SYM_REGEX$|ANY_SUFFIX$|EOF$')
     static_analysis_pending__ = []  # type: List[bool]
     parser_initialization__ = ["upon instantiation"]
@@ -143,7 +143,7 @@ class FixedEBNFGrammar(Grammar):
     root__ = syntax
     
 
-_raw_grammar = ThreadLocalSingletonFactory(FixedEBNFGrammar, ident=1)
+_raw_grammar = ThreadLocalSingletonFactory(FixedEBNFGrammar)
 
 def get_grammar() -> FixedEBNFGrammar:
     grammar = _raw_grammar()
