@@ -94,7 +94,7 @@ class FlexibleEBNFGrammar(Grammar):
     countable = Forward()
     element = Forward()
     expression = Forward()
-    source_hash__ = "013a2921fe249573eede3c87a5ebb192"
+    source_hash__ = "849fcfe8b743d4f0c4441a8eabc01c66"
     disposable__ = re.compile('component$|pure_elem$|countable$|FOLLOW_UP$|SYM_REGEX$|ANY_SUFFIX$|EOF$')
     static_analysis_pending__ = []  # type: List[bool]
     parser_initialization__ = ["upon instantiation"]
@@ -167,7 +167,7 @@ class FlexibleEBNFGrammar(Grammar):
     root__ = syntax
     
 
-_raw_grammar = ThreadLocalSingletonFactory(FlexibleEBNFGrammar, ident=1)
+_raw_grammar = ThreadLocalSingletonFactory(FlexibleEBNFGrammar)
 
 def get_grammar() -> FlexibleEBNFGrammar:
     grammar = _raw_grammar()
