@@ -92,7 +92,7 @@ class LaTeXGrammar(Grammar):
     paragraph = Forward()
     param_block = Forward()
     tabular_config = Forward()
-    source_hash__ = "ff43ac15aba174b3001eb07c5058ba9d"
+    source_hash__ = "e7f9979dd243430578b4f2ddafc33a5a"
     disposable__ = re.compile('_\\w+')
     static_analysis_pending__ = []  # type: List[bool]
     parser_initialization__ = ["upon instantiation"]
@@ -263,7 +263,7 @@ class LaTeXGrammar(Grammar):
     root__ = TreeReduction(latexdoc, CombinedParser.MERGE_TREETOPS)
     
 
-_raw_grammar = ThreadLocalSingletonFactory(LaTeXGrammar, ident=1)
+_raw_grammar = ThreadLocalSingletonFactory(LaTeXGrammar)
 
 def get_grammar() -> LaTeXGrammar:
     grammar = _raw_grammar()
