@@ -47,7 +47,7 @@ from DHParser import start_logging, suspend_logging, resume_logging, is_filename
     has_attr, has_parent, ThreadLocalSingletonFactory, Error, canonical_error_strings, \
     has_errors, ERROR, FATAL, set_preset_value, get_preset_value, NEVER_MATCH_PATTERN, \
     gen_find_include_func, preprocess_includes, make_preprocessor, chain_preprocessors, \
-    PLACEHOLDER
+    PLACEHOLDER, line_col
 
 
 #######################################################################
@@ -93,7 +93,7 @@ class XMLGrammar(Grammar):
     r"""Parser for a XML source file.
     """
     element = Forward()
-    source_hash__ = "8aec099040a4deac970f5ad623755b3e"
+    source_hash__ = "72a5667a6a815d969078d588906cd565"
     disposable__ = re.compile('BOM$|Misc$|NameStartChar$|NameChars$|CommentChars$|PubidChars$|prolog$|PubidCharsSingleQuoted$|VersionNum$|EncName$|Reference$|CData$|EOF$|tagContent$')
     static_analysis_pending__ = []  # type: List[bool]
     parser_initialization__ = ["upon instantiation"]
@@ -151,7 +151,7 @@ class XMLGrammar(Grammar):
     root__ = TreeReduction(document, CombinedParser.MERGE_TREETOPS)
     
 
-_raw_grammar = ThreadLocalSingletonFactory(XMLGrammar, ident=1)
+_raw_grammar = ThreadLocalSingletonFactory(XMLGrammar)
 
 def get_grammar() -> XMLGrammar:
     grammar = _raw_grammar()
@@ -268,7 +268,10 @@ class XMLTransformer(Compiler):
 
     def on_element(self, node):
         if len(node.children) == 1:
-            return self.on_emptyElement(node['emptyElement'])
+            try:
+                return self.on_emptyElement(node['emptyElement'])
+            except KeyError:  # in case of a malformed tree...
+                return node[0]
         stag = node['STag']
         etag = node['ETag']
         tag_name = stag['Name'].content
@@ -317,7 +320,7 @@ class XMLTransformer(Compiler):
         return node
 
     def on_Reference(self, node):
-        replace_by_single_child(self.trail)
+        replace_by_single_child(self.path)
         return node
 
     def on_Comment(self, node):
