@@ -77,7 +77,7 @@ class XML_DTDGrammar(Grammar):
     element = Forward()
     extSubsetDecl = Forward()
     ignoreSectContents = Forward()
-    source_hash__ = "cd1662e4b959ba3594ba074c594c54e9"
+    source_hash__ = "b5dd82adc12c32aefc3629d644a5f0f3"
     disposable__ = re.compile('..(?<=^)')
     static_analysis_pending__ = []  # type: List[bool]
     parser_initialization__ = ["upon instantiation"]
@@ -184,7 +184,7 @@ class XML_DTDGrammar(Grammar):
     root__ = document
     
 
-_raw_grammar = ThreadLocalSingletonFactory(XML_DTDGrammar, ident=1)
+_raw_grammar = ThreadLocalSingletonFactory(XML_DTDGrammar)
 
 def get_grammar() -> XML_DTDGrammar:
     grammar = _raw_grammar()
