@@ -97,7 +97,7 @@ class XML_W3C_SPECGrammar(Grammar):
     element = Forward()
     extSubsetDecl = Forward()
     ignoreSectContents = Forward()
-    source_hash__ = "849389162fff5129c9f2101567f242da"
+    source_hash__ = "ac5909c47cac99f289d336638ae4fc5d"
     disposable__ = re.compile('..(?<=^)')
     static_analysis_pending__ = []  # type: List[bool]
     parser_initialization__ = ["upon instantiation"]
@@ -188,7 +188,7 @@ class XML_W3C_SPECGrammar(Grammar):
     root__ = document
     
 
-_raw_grammar = ThreadLocalSingletonFactory(XML_W3C_SPECGrammar, ident=1)
+_raw_grammar = ThreadLocalSingletonFactory(XML_W3C_SPECGrammar)
 
 def get_grammar() -> XML_W3C_SPECGrammar:
     grammar = _raw_grammar()
