@@ -130,10 +130,10 @@ class TestCompilerGeneration:
             compiler_suite_2nd_run = f.read()
         diff = ''.join([a for a, b in zip(compiler_suite_2nd_run, compiler_suite) if a != b])
         # differences should only be ID-Numbers
-        assert diff.isnumeric()
+        assert not diff or diff.isnumeric()
         # test compiling with a generated compiler suite
         # assert is_filename(self.text_name)
-        errors = compile_on_disk(self.text_name, self.compiler_name)
+        errors = compile_on_disk(self.text_name, compiler_suite=self.compiler_name)
         assert not errors, str(errors)
         assert os.path.exists(self.result_name)
         with open(self.result_name, 'r') as f:
