@@ -29,10 +29,10 @@ sys.path.append(os.path.abspath(os.path.join(scriptpath, '..')))
 
 from DHParser.configuration import get_config_value, set_config_value
 from DHParser.nodetree import Node, RootNode, parse_sxpr, parse_xml, flatten_sxpr, \
-    flatten_xml, parse_json, ZOMBIE_TAG, EMPTY_NODE, ANY_NODE, next_trail, \
-    prev_trail, serialize_trail, generate_content_mapping, map_pos_to_trail, \
-    select_trail_if, select_trail, create_trail_match_function, pick_trail, \
-    LEAF_TRAIL, TOKEN_PTYPE, insert_node, markup, content_of, strlen_of
+    flatten_xml, parse_json, ZOMBIE_TAG, EMPTY_NODE, ANY_NODE, next_path, \
+    prev_path, pick_from_path, pp_path, ContentMapping, leaf_paths, NO_PATH, \
+    select_path_if, select_path, create_path_match_function, pick_path, \
+    LEAF_PATH, TOKEN_PTYPE, insert_node, content_of, strlen_of, gen_chain_ID
 from DHParser.preprocess import gen_neutral_srcmap_func
 from DHParser.transform import traverse, reduce_single_child, \
     replace_by_single_child, flatten, remove_empty, remove_whitespace
@@ -105,7 +105,7 @@ class TestParseSxpression:
         tree.new_error(tree, 'Fehler, "mein Herr"!', ERROR)
         assert tree.equals(parse_sxpr('(u "XXX")'))
         sxpr = tree.as_sxpr()
-        assert sxpr == r'''(u `(err "Error (1000): Fehler, \"mein Herr\"!") "XXX")'''
+        assert sxpr == r'''(u `(err "Error (1000): Fehler, \"mein Herr\"!") "XXX")''', sxpr
         tree2 = parse_sxpr(sxpr)
         assert tree2.as_sxpr() == r'''(u `(err "Error (1000): Fehler, \"mein Herr\"!") "XXX")'''
         # When backparsing s-expression-serialized trees, errors are parsed as attributes
@@ -175,7 +175,7 @@ class TestParseXML:
         tree = parse_xml('<a>alpha <b>beta</b> gamma</a>')
         assert flatten_sxpr(tree.as_sxpr()) == '(a (:Text "alpha ") (b "beta") (:Text " gamma"))'
         tree = parse_xml(' <a>  <b>beta</b>  </a> ')
-        assert flatten_xml(tree.as_xml()) == \
+        assert flatten_xml(tree.as_xml(string_tags=set())) == \
                '<a><ANONYMOUS_Text__>  </ANONYMOUS_Text__><b>beta</b>' \
                '<ANONYMOUS_Text__>  </ANONYMOUS_Text__></a>'
         assert tree.as_xml(inline_tags={'a'}, string_tags={':Text'}) == '<a>  <b>beta</b>  </a>'
@@ -421,20 +421,20 @@ class TestNode:
                 for node in self.unique_tree.select(ANY_NODE, include_root=True, skip_subtree='f')]
         assert ''.join(tags) == "abdf", ''.join(tags)
 
-    def test_tree_select_trail_if(self):
+    def test_tree_select_path_if(self):
         tree = parse_sxpr(self.unique_nodes_sexpr)
         contexts = []
-        for ctx in tree.select_trail_if(lambda ctx: ctx[-1].name >= 'd',
-                                        include_root=True, reverse=False):
+        for ctx in tree.select_path_if(lambda ctx: ctx[-1].name >= 'd',
+                                       include_root=True, reverse=False):
             contexts.append(''.join(nd.name for nd in ctx))
         assert contexts == ['ad', 'af', 'afg']
         contexts = []
-        for ctx in tree.select_trail_if(lambda ctx: ctx[-1].name >= 'd',
-                                        include_root=True, reverse=True):
+        for ctx in tree.select_path_if(lambda ctx: ctx[-1].name >= 'd',
+                                       include_root=True, reverse=True):
             contexts.append(''.join(nd.name for nd in ctx))
         assert contexts == ['af', 'afg', 'ad']
 
-    def test_select_trail_with_skipping(self):
+    def test_select_path_with_skipping(self):
         tree = parse_sxpr('(a (b c) (d e) (f (g h)))')
         check = []
         contexts = []
@@ -442,20 +442,20 @@ class TestNode:
             nonlocal check
             check.append(''.join(nd.name for nd in ctx))
             return True
-        for ctx in tree.select_trail(select_f, include_root=True):
+        for ctx in tree.select_path(select_f, include_root=True):
             contexts.append(''.join(nd.name for nd in ctx))
         assert check == contexts == ['a', 'ab', 'ad', 'af', 'afg']
         check = []
         contexts = []
-        for ctx in tree.select_trail(select_f, include_root=True, skip_subtree='f'):
+        for ctx in tree.select_path(select_f, include_root=True, skip_subtree='f'):
             contexts.append(''.join(nd.name for nd in ctx))
         assert check == contexts == ['a', 'ab', 'ad', 'af']
 
-    def test_tree_select_trail(self):
+    def test_tree_select_path(self):
         tree = parse_sxpr('(A (B 1) (C (X 1) (Y 1)) (B 2))')
         l = [str(nd) for nd in tree.select('B')]
         assert ''.join(l) == '12'
-        l = [str(ctx[-1]) for ctx in tree.select_trail('B')]
+        l = [str(ctx[-1]) for ctx in tree.select_path('B')]
         assert ''.join(l) == '12'
 
     def test_select_children(self):
@@ -543,16 +543,16 @@ class TestNode:
         t2 = copy.deepcopy(tree_copy)
         res2 = compiler(t2)
         diff = ''.join([a for a, b in zip(res1, res2) if a != b])
-        assert diff.isnumeric()  # differences should only be ID-Numbers
+        assert not diff or diff.isnumeric()  # differences should at most be ID-Numbers
         tree_copy = copy.deepcopy(tree)
         transform(tree_copy)
         res3 = compiler(tree_copy)
         diff = ''.join([a for a, b in zip(res2, res3) if a != b])
-        assert diff.isnumeric()  # differences should only be ID-Numbers
+        assert not diff or diff.isnumeric()  # differences should only be ID-Numbers
         transform(tree)
         res4 = compiler(tree)
         diff = ''.join([a for a, b in zip(res3, res4) if a != b])
-        assert diff.isnumeric()  # differences should only be ID-Numbers
+        assert not diff or diff.isnumeric()  # differences should only be ID-Numbers
 
     def test_len_and_pos(self):
         """Test len-property of Node."""
@@ -851,14 +851,16 @@ class TestSerialization:
 
     def test_XML_strict_mode(self):
         xml = "<xml><empty/><empty>not_empty!?</empty></xml>"
-        try:
-            data = parse_xml(xml)
-            assert False, "ValueError exptected!"
-        except ValueError:
-            pass
+        # try:
+        #     data = parse_xml(xml)
+        #     assert False, "ValueError exptected!"
+        # except ValueError:
+        #     pass
         empty_tags = set()
         data = parse_xml(xml, out_empty_tags=empty_tags, strict_mode=False)
-        assert data.as_sxpr() == '(xml (empty) (empty "not_empty!?"))'
+        for e in data.errors:
+            print(e)
+        # assert data.as_sxpr() == '(xml (empty) (empty "not_empty!?"))'
         assert empty_tags == {'empty'}
         assert data.as_xml().replace('\n', '').replace(' ', '') == \
             "<xml><empty></empty><empty>not_empty!?</empty></xml>"
@@ -877,29 +879,29 @@ class TestSerialization:
 
 
 class TestSegementExtraction:
-    def test_get_trail(self):
+    def test_get_path(self):
         tree = parse_sxpr('(A (F (X "a") (Y "b")) (G "c"))')
         nd_X = tree.pick('X')
-        ctx = tree.reconstruct_trail(nd_X)
+        ctx = tree.reconstruct_path(nd_X)
         assert [nd.name for nd in ctx] == ['A', 'F', 'X']
         nd_F = tree.pick('F')
         nd_Y = tree.pick('Y')
-        ctx = nd_F.reconstruct_trail(nd_Y)
+        ctx = nd_F.reconstruct_path(nd_Y)
         assert [nd.name for nd in ctx] == ['F', 'Y']
-        ctx = tree.reconstruct_trail(nd_F)
+        ctx = tree.reconstruct_path(nd_F)
         assert [nd.name for nd in ctx] == ['A', 'F']
-        ctx = tree.reconstruct_trail(nd_Y)
+        ctx = tree.reconstruct_path(nd_Y)
         assert [nd.name for nd in ctx] == ['A', 'F', 'Y']
         nd_G = tree.pick('G')
-        ctx = tree.reconstruct_trail(nd_G)
+        ctx = tree.reconstruct_path(nd_G)
         assert [nd.name for nd in ctx] == ['A', 'G']
         not_there = Node('not_there', '')
         try:
-            tree.reconstruct_trail(not_there)
+            tree.reconstruct_path(not_there)
             assert False, "ValueError expected!"
         except ValueError:
             pass
-        assert tree.reconstruct_trail(tree) == [tree]
+        assert tree.reconstruct_path(tree) == [tree]
 
     def test_milestone_segment(self):
         tree = parse_sxpr('(root (left (A "a") (B "b") (C "c")) (middle "-") (right (X "x") (Y "y") (Z "z")))').with_pos(0)
@@ -950,56 +952,56 @@ class TestPositionAssignment:
         assert (tree['G']['N'].pos, tree['G']['P'].pos) == (1, 5)
 
 
-class TestTrailNavigation:
+class TestPathNavigation:
     tree = parse_sxpr('(A (B 1) (C (D 2) (E 3)) (F 4))')
 
-    def test_prev_trail(self):
-        ctx = self.tree.pick_trail('D')
-        c = prev_trail(ctx)
+    def test_prev_path(self):
+        ctx = self.tree.pick_path('D')
+        c = prev_path(ctx)
         assert c[-1].name == 'B'
-        ctx = self.tree.pick_trail('E')
-        c = prev_trail(ctx)
+        ctx = self.tree.pick_path('E')
+        c = prev_path(ctx)
         assert c[-1].name == 'D'
-        ctx = self.tree.pick_trail('F')
-        c = prev_trail(ctx)
+        ctx = self.tree.pick_path('F')
+        c = prev_path(ctx)
         assert c[-1].name == 'C'
-        ctx = self.tree.pick_trail('C')
-        c = prev_trail(ctx)
+        ctx = self.tree.pick_path('C')
+        c = prev_path(ctx)
         assert c[-1].name == 'B'
 
-    def test_next_trail(self):
-        ctx = self.tree.pick_trail('D')
-        c = next_trail(ctx)
+    def test_next_path(self):
+        ctx = self.tree.pick_path('D')
+        c = next_path(ctx)
         assert c[-1].name == 'E'
-        ctx = self.tree.pick_trail('E')
-        c = next_trail(ctx)
+        ctx = self.tree.pick_path('E')
+        c = next_path(ctx)
         assert c[-1].name == 'F'
-        ctx = self.tree.pick_trail('C')
-        c = next_trail(ctx)
+        ctx = self.tree.pick_path('C')
+        c = next_path(ctx)
         assert c[-1].name == 'F'
-        ctx = self.tree.pick_trail('B')
-        c = next_trail(ctx)
+        ctx = self.tree.pick_path('B')
+        c = next_path(ctx)
         assert c[-1].name == 'C'
 
     def test_content_mapping(self):
         tree = parse_sxpr('(A (B alpha) (C (D beta) (E gamma)) (F delta))')
-        cm = generate_content_mapping(tree)
+        cm = ContentMapping(tree)
         for i in range(len(tree.content)):
-            ctx, rel_pos = map_pos_to_trail(i, cm)
-        i = tree.content.find("delta")
-        ctx, rel_pos = map_pos_to_trail(i, cm)
+            ctx, rel_pos = cm.get_path_and_offset(i)
+        i = cm.content.find("delta")
+        ctx, rel_pos = cm.get_path_and_offset(i)
         assert ctx[-1].name == 'F' and rel_pos == 0
-        i = tree.content.find("lta")
-        ctx, rel_pos = map_pos_to_trail(i, cm)
+        i = cm.content.find("lta")
+        ctx, rel_pos = cm.get_path_and_offset(i)
         assert ctx[-1].name == 'F' and rel_pos == 2
-        i = tree.content.find("a")
-        ctx, rel_pos = map_pos_to_trail(i, cm)
+        i = cm.content.find("a")
+        ctx, rel_pos = cm.get_path_and_offset(i)
         assert ctx[-1].name == 'B' and rel_pos == 0
-        i = tree.content.rfind("a")
-        ctx, rel_pos = map_pos_to_trail(i, cm)
+        i = cm.content.rfind("a")
+        ctx, rel_pos = cm.get_path_and_offset(i)
         assert ctx[-1].name == 'F' and rel_pos == 4
         i = tree.content.find("mm")
-        ctx, rel_pos = map_pos_to_trail(i, cm)
+        ctx, rel_pos = cm.get_path_and_offset(i)
         assert ctx[-1].name == 'E' and rel_pos == 2
 
         select, ignore = {'A', 'B', 'D', 'F'}, {'C'}
@@ -1011,33 +1013,45 @@ class TestTrailNavigation:
         assert content_of(tree, select, ignore | {'A'}) == ""
         assert strlen_of(tree, select, ignore | {'A'}) == 0
 
-        mapping = generate_content_mapping(tree, select, ignore)
-        insert_node(mapping, content.find('delta'),
-                    Node('G', 'omicron'))
+        mapping = ContentMapping(tree, leaf_paths(select), ignore)
+        mapping.insert_node(content.find('delta'), Node('G', 'omicron'))
         assert flatten_sxpr(tree.as_sxpr()) == \
                '(A (B "alpha") (C (D "beta") (E "gamma")) (G "omicron") (F "delta"))'
 
-    def test_standalone_select_trail_if(self):
-        start = self.tree.pick_trail('E')
+    def test_map_index(self):
+        tree = parse_xml('<doc>alpha<a><lb/></a><pb/> beta</doc>')
+        cm = ContentMapping(tree)
+        assert cm.get_path_index(5, left_biased=False) == 3
+        assert cm.get_path_index(5, left_biased=True) == 0
+        path, i = cm.get_path_and_offset(5, left_biased=False)
+        assert path == cm.path_list[3]
+        assert i == 0
+        path, i = cm.get_path_and_offset(5, left_biased=True)
+        assert path == cm.path_list[0]
+        assert i == 5
+
+
+    def test_standalone_select_path_if(self):
+        start = self.tree.pick_path('E')
         save = start.copy()
         sequence = []
-        for ctx in select_trail_if(
+        for ctx in select_path_if(
                 start, lambda c: True, include_root=True, reverse=True):
             sequence.append(''.join(n.name for n in ctx))
         assert sequence == ['ACE', 'ACD', 'AC', 'AB', 'A']
-        assert save == start  # trail passed should not be changed by select_trail
+        assert save == start  # path passed should not be changed by select_path
 
-        start = self.tree.pick_trail('D')
+        start = self.tree.pick_path('D')
         sequence = []
-        for ctx in select_trail_if(
+        for ctx in select_path_if(
                 start, lambda c: True, include_root=True, reverse=False):
             sequence.append(''.join(n.name for n in ctx))
         assert sequence == ['ACD', 'ACE', 'AC', 'AF', 'A']
 
-    def test_standalone_pick_trail(self):
-        start = self.tree.pick_trail('A', include_root=True)
-        anfang = pick_trail(start, LEAF_TRAIL, include_root=True)
-        ende = pick_trail(start, LEAF_TRAIL, include_root=True, reverse=True)
+    def test_standalone_pick_path(self):
+        start = self.tree.pick_path('A', include_root=True)
+        anfang = pick_path(start, LEAF_PATH, include_root=True)
+        ende = pick_path(start, LEAF_PATH, include_root=True, reverse=True)
         assert anfang[-1].name == 'B'
         assert ende[-1].name == 'F'
 
@@ -1104,12 +1118,71 @@ class TestMarkupInsertion:
 </app> glandiferae</document>'''
     testdata_3 = '''<doc>Am <outer><inner>Anfang</inner> war das Wort</outer>.</doc>'''
 
-    def test_content(self):
+    def test_content_of(self):
         tree = parse_xml('<p>This is<fn>footnote</fn> a text</p>')
         assert content_of(tree) == 'This isfootnote a text'
         assert content_of(tree, ignore='fn') == 'This is a text'
         assert content_of(tree, select='fn') == 'footnote'
 
+    def test_ContentMapping_constructor(self):
+        tree = parse_xml('<doc><p>In München<footnote><em>München</em> is the '
+            'German name of the city of Munich</footnote> is a Hofbräuhaus</p></doc>')
+        try:
+            cm = ContentMapping(tree, select='footnote')
+            assert False, "ValueError expected"
+        except ValueError as e:
+            pass
+        cm = ContentMapping(tree, select=lambda pth: pick_from_path(pth, 'footnote')
+                                                     and not pth[-1].children)
+
+    def test_ContentMapping_rebuild_mapping(self):
+        tree = parse_xml('<doc><p>In München<footnote><em>München</em> is the '
+            'German name of the city of Munich</footnote> is a Hofbräuhaus</p></doc>')
+        fm = ContentMapping(tree, select=leaf_paths('footnote'), ignore=NO_PATH)
+        i = fm.content.find('München')
+        path, offset = fm.get_path_and_offset(i)
+        path[-1].result = path[-1].result[:offset] + "Stadt " + path[-1].result[offset:]
+        assert tree.as_xml(inline_tags={'doc'}) == ('<doc><p>In München<footnote>'
+            '<em>Stadt München</em> is the German name of the city of Munich</footnote> '
+            'is a Hofbräuhaus</p></doc>')
+        k = fm.get_path_index(i)
+        fm.rebuild_mapping_slice(k, k)
+        assert fm.pos_list == [0, 13]
+
+    def test_chain_id(self):
+        s = set()
+        for i in range(200_000):
+            cid = gen_chain_ID()
+            assert cid not in s
+            s.add(cid)
+            assert len(cid) <= 4
+        assert len(s) == 200_000
+
+    def test_chain_attr_1(self):
+        tree = parse_xml("<hard>Please mark up Stadt\n<lb/><location><em>München</em> "
+                         "in Bavaria</location> in this sentence.</hard>")
+        divisability_map = {'foreign': {'location', ':Text'},
+                                        '*': {':Text'}}
+        mapping = ContentMapping(tree, divisability=divisability_map,
+                                 chain_attr_name = "chain")
+        match = re.search(r"Stadt\s+München", mapping.content)
+        _ = mapping.markup(match.start(), match.end(), "foreign", {'lang': 'de'})
+        xml_str = tree.as_xml(empty_tags={'lb'})
+        # print(xml_str)
+        chains = {loc.attr['chain'] for loc in tree.select('location')}
+        assert len(chains) == 1
+
+    # def test_chain_attr_2(self):
+    #     tree = parse_xml("<doc>Please mark up Stadt\n<lb/>"
+    #         "<em>München</em><footnote>'Stadt <em>München</em>'"
+    #         " is German for 'City of Munich'</footnote> in Bavaria"
+    #         " in this sentence.</doc>")
+    #     cm = ContentMapping(tree, select=LEAF_PATH, ignore='footnote')
+    #     m = re.search(r"München\s+in\s+Bavaria", cm.content)
+    #     _ = cm.markup(m.start(), m.end(), 'location', chain_attr_name='chain')
+    #     print(tree.as_xml(empty_tags={'lb'}))
+
+
     def test_insert_milestone_1(self):
         empty_tags = set()
         tree = parse_xml(self.testdata_1, string_tag=TOKEN_PTYPE, out_empty_tags=empty_tags)
@@ -1117,8 +1190,8 @@ class TestMarkupInsertion:
         assert i >= 0
         milestone = Node("ref", "").with_attr(type="subj", target="Charlottenburg_S00231")
         empty_tags.add("ref")
-        tm = generate_content_mapping(tree)
-        insert_node(tm, i, milestone)
+        tm = ContentMapping(tree)
+        tm.insert_node(i, milestone)
         xml = tree.as_xml(inline_tags={"document"}, string_tags={TOKEN_PTYPE},
                           empty_tags=empty_tags)
         assert xml == ('<document>In <ref type="subj" '
@@ -1131,8 +1204,8 @@ class TestMarkupInsertion:
         m = next(re.finditer(r'silvae,?\s*glandiferae', tree.content))
         milestone = Node("ref", "").with_attr(type="subj", target="silva_glandifera_S01229")
         empty_tags.add("ref")
-        tm = generate_content_mapping(tree)
-        insert_node(tm, m.start(), milestone)
+        tm = ContentMapping(tree)
+        tm.insert_node(m.start(), milestone)
         xml = tree.as_xml(inline_tags={"document"}, string_tags={TOKEN_PTYPE},
                           empty_tags=empty_tags)
         assert xml == ('<document><app n="g"><lem>silvae</lem><ref type="subj" '
@@ -1145,8 +1218,8 @@ class TestMarkupInsertion:
         i = tree.content.find("Charlottenburg")
         assert i >= 0
         k = i + len("Charlottenburg")
-        tm = generate_content_mapping(tree)
-        markup(tm, i, k, "ref", { 'type':"subj", 'target':"Charlottenburg_S00231"})
+        tm = ContentMapping(tree)
+        tm.markup(i, k, "ref", { 'type':"subj", 'target':"Charlottenburg_S00231"})
         xml = tree.as_xml(inline_tags={"document"}, string_tags={TOKEN_PTYPE},
                           empty_tags=empty_tags)
         assert xml == ('<document>In <ref type="subj" '
@@ -1157,8 +1230,8 @@ class TestMarkupInsertion:
         empty_tags = set()
         tree = parse_xml(self.testdata_2, string_tag=TOKEN_PTYPE, out_empty_tags=empty_tags)
         m = next(re.finditer(r'silvae,?\s*glandiferae', tree.content))
-        tm = generate_content_mapping(tree)
-        markup(tm, m.start(), m.end(), "ref", {'type': "subj", 'target': "silva_glandifera_S01229"})
+        tm = ContentMapping(tree)
+        tm.markup(m.start(), m.end(), "ref", {'type': "subj", 'target': "silva_glandifera_S01229"})
         xml = tree.as_xml(inline_tags={"document"}, string_tags={TOKEN_PTYPE},
                           empty_tags=empty_tags)
         assert xml == ('<document><app n="g"><lem>silvae</lem><ref type="subj" '
@@ -1169,18 +1242,92 @@ class TestMarkupInsertion:
         empty_tags = set()
         tree = parse_xml(self.testdata_3, string_tag=TOKEN_PTYPE, out_empty_tags=empty_tags)
         m = next(re.finditer(r'Anfang war das Wort', tree.content))
-        cm = generate_content_mapping(tree)
-        markup(cm, m.start(), m.end(), "a")
+        cm = ContentMapping(tree)
+        cm.markup(m.start(), m.end(), "a")
         assert tree.as_xml(inline_tags={'doc'}, string_tags={':Text'}) == \
             '<doc>Am <outer><a><inner>Anfang</inner> war das Wort</a></outer>.</doc>'
         empty_tags = set()
         tree = parse_xml(self.testdata_3, string_tag=TOKEN_PTYPE, out_empty_tags=empty_tags)
         m = next(re.finditer(r'Am Anfang war', tree.content))
-        cm = generate_content_mapping(tree)
-        markup(cm, m.start(), m.end(), "a")
+        cm = ContentMapping(tree)
+        cm.markup(m.start(), m.end(), "a")
         assert tree.as_xml(inline_tags={'doc'}, string_tags={':Text'}) == \
             '<doc><a>Am </a><outer><a><inner>Anfang</inner> war</a> das Wort</outer>.</doc>'
 
+    def test_insert_markup_4(self):
+        empty_tags = set()
+        tree = parse_xml(self.testdata_3, string_tag=TOKEN_PTYPE, out_empty_tags=empty_tags)
+        m = next(re.finditer(r'Am Anfang war', tree.content))
+        cm = ContentMapping(tree, chain_attr_name='_chain')
+        cm.markup(m.start(), m.end(), "a")
+        I = tree.pick('a').attr['_chain']
+        assert tree.as_xml(inline_tags={'doc'}, string_tags={':Text'}) == \
+            f'<doc><a _chain="{I}">Am </a><outer><a _chain="{I}"><inner>Anfang</inner> war</a>'\
+            f' das Wort</outer>.</doc>'
+
+    def test_insert_markup_5(self):
+        tree = parse_xml('<text><hi rend="i">X</hi>34, 53 ... Q. Aelius Tubero tribunus plebis</text>')
+        i = tree.content.find('Q.')
+        k = tree.content.find('Tubero') + len('Tubero')
+        cm = ContentMapping(tree)
+        cm.markup(i, k, "a")
+        assert tree.as_xml(inline_tags={'text'}, string_tags={TOKEN_PTYPE}) == \
+            '<text><hi rend="i">X</hi>34, 53 ... <a>Q. Aelius Tubero</a> tribunus plebis</text>'
+
+    def test_insert_markup_6(self):
+        tree = parse_xml('<doc>wenn wir bei <hi rend="italic">Cicero</hi><note type="footnote" n="29)">'
+            '<pb n="225"/>In Verr. acc. 1. 3, 120. </note> die meist nicht erhebli<lb/>chen Zahlen</doc>')
+        m = re.search(r'Cicero', tree.content)
+        a, b = m.start(), m.end()
+        cm = ContentMapping(tree, auto_cleanup=True)
+        cm.markup(a, b, 'ref')
+        assert flatten_sxpr(tree.as_sxpr()) == '(doc (:Text "wenn wir bei ") '\
+            '(hi `(rend "italic") (ref "Cicero")) (note `(type "footnote") `(n "29)") '\
+            '(pb `(n "225")) (:Text "In Verr. acc. 1. 3, 120. ")) '\
+            '(:Text " die meist nicht erhebli") (lb) (:Text "chen Zahlen"))'
+
+    def test_insert_markup_7(self):
+        tree= parse_xml('<doc>Zugleich traf sie für <pb n="219"/>den ager compascuus folgende Bestimmung '
+            '(Z. 14, 15 nach <hi rend="italic">Momm</hi><lb/><hi rend="italic">sens</hi>'
+            '<note type="comment" n="53"><pb n="219"/>Offenbar</note> nach:</doc>')
+        m = re.search(r'Mommsen', tree.content)
+        a, b = m.start(), m.end()
+        cm = ContentMapping(tree, auto_cleanup=True)
+        cm.markup(a, b, 'ref')
+        assert flatten_sxpr(tree.as_sxpr()) == '(doc (:Text "Zugleich traf sie für ") '\
+            '(pb `(n "219")) (:Text "den ager compascuus folgende Bestimmung '\
+            '(Z. 14, 15 nach ") (ref (hi `(rend "italic") "Momm") (lb)) (hi `(rend "italic") '\
+            '(ref "sen") (:Text "s")) (note `(type "comment") `(n "53") (pb `(n "219")) '\
+            '(:Text "Offenbar")) (:Text " nach:"))'
+
+    def test_insert_markup_8(self):
+        tree = parse_xml('<item><pb n="283" ed="A"></pb><hi rend="italic">Beaudouin</hi>. Études'
+            '<note type="comment" n="10">Im Titel heißt es: Étude. </note> sur le jus Italicum '
+            '(Nouvelle revue historique V, 1881, p. 145ff.). </item>')
+        m = re.search('Beaudouin. Études', tree.content)
+        a, b = m.start(), m.end()
+        cm = ContentMapping(tree, auto_cleanup=True)
+        cm.markup(a, b, 'ref')
+        assert tree.as_xml(inline_tags={'item'}) == '<item><pb n="283" ed="A"></pb>'\
+            '<ref><hi rend="italic">Beaudouin</hi>. Études</ref><note type="comment" n="10">'\
+            'Im Titel heißt es: Étude. </note> sur le jus Italicum (Nouvelle revue historique '\
+            'V, 1881, p. 145ff.). </item>'
+
+    def test_maskup_borderline_cases(self):
+        ### borderline cases
+        tree = parse_xml('<doc>Hello, <em>World</em>!</doc>')
+        X = copy.deepcopy(tree)
+        t = ContentMapping(X)
+        _ = t.markup(3, 4, 'b')
+        assert X.as_xml(inline_tags={'doc'}) \
+            == '<doc>Hel<b>l</b>o, <em>World</em>!</doc>'
+
+        X = copy.deepcopy(tree)
+        t = ContentMapping(X)
+        _ = t.markup(3, 3, 'b')
+        assert X.as_xml(inline_tags={'doc'}, empty_tags={'b'}) \
+               == '<doc>Hel<b/>lo, <em>World</em>!</doc>'
+
 
 if __name__ == "__main__":
     from DHParser.testing import runner
