@@ -27,14 +27,14 @@ scriptpath = os.path.dirname(__file__) or '.'
 sys.path.append(os.path.abspath(os.path.join(scriptpath, '..')))
 scriptpath = os.path.abspath(scriptpath)
 
-from DHParser.syntaxtree import parse_sxpr, flatten_sxpr, TOKEN_PTYPE
+from DHParser.nodetree import parse_sxpr, flatten_sxpr, TOKEN_PTYPE
 from DHParser.transform import traverse, remove_whitespace, remove_empty, \
     replace_by_single_child, reduce_single_child, flatten, add_error
-from DHParser.dsl import grammar_provider
-from DHParser.error import Error, PARSER_LOOKAHEAD_FAILURE_ONLY, PARSER_LOOKAHEAD_MATCH_ONLY, \
-    MANDATORY_CONTINUATION_AT_EOF, ERROR, WARNING
+from DHParser.dsl import grammar_provider, create_parser
+from DHParser.error import PARSER_LOOKAHEAD_FAILURE_ONLY, PARSER_LOOKAHEAD_MATCH_ONLY, \
+    MANDATORY_CONTINUATION_AT_EOF, MANDATORY_CONTINUATION_AT_EOF_NON_ROOT, ERROR
 from DHParser.testing import get_report, grammar_unit, unit_from_file, \
-    clean_report, unique_name
+    unit_from_config, clean_report, unique_name, reset_unit
 from DHParser.trace import set_tracer, trace_history
 
 
@@ -149,7 +149,6 @@ class TestTestfiles:
             pass
 
 
-
 ARITHMETIC_EBNF = """
     @ whitespace = vertical
     @ literalws = right
@@ -171,7 +170,7 @@ ARITHMETIC_EBNF_transformation_table = {
 }
 
 
-ARITHMETIC_EBNFTransform = partial(traverse, processing_table=ARITHMETIC_EBNF_transformation_table)
+ARITHMETIC_EBNFTransform = partial(traverse, transformation_table=ARITHMETIC_EBNF_transformation_table)
 
 
 class TestGrammarTest:
@@ -241,11 +240,16 @@ class TestGrammarTest:
         trans_fac = lambda : ARITHMETIC_EBNFTransform
         # reset_unit(self.cases)
         errata = grammar_unit(self.cases, parser_fac, trans_fac, 'REPORT_TestGrammarTest')
-        assert errata, "Unknown parser, but no error message!?"
+        assert len(errata) == 1
+        assert errata[0] == 'Unknown parser "no_match_tests_specified" in fail test "1"!', \
+            "Unknown parser, but no error message!?"
         report = get_report(self.cases)
         assert report.find('### CST') >= 0
         errata = grammar_unit(self.failure_cases, parser_fac, trans_fac, 'REPORT_TestGrammarTest')
         assert len(errata) == 3, str(errata)
+        assert errata[0].find('Match test "1"') >= 0
+        assert errata[1].find('Abstract syntax tree test "3"') >= 0
+        assert errata[2].find('Fail test "4"') >= 0
 
     def test_fail_failtest(self):
         """Failure test should not pass if it failed because the parser is unknown."""
@@ -292,8 +296,9 @@ class TestASTErrors:
 
     def test_errors_added_during_AST_transformation(self):
         parser_fac = grammar_provider(ARITHMETIC_EBNF)
-        trans_fac = lambda : partial(traverse, processing_table=self.trans_table)
+        trans_fac = lambda : partial(traverse, transformation_table=self.trans_table)
         errata = grammar_unit(self.cases1, parser_fac, trans_fac, 'REPORT_ASTFailureTest')
+        assert len(errata) == 1
         assert "marked with error" in str(errata)
         errata = grammar_unit(self.cases2, parser_fac, trans_fac, 'REPORT_ASTFailureTest')
         assert not errata
@@ -343,7 +348,7 @@ class TestLookahead:
     }
 
     grammar_fac = grammar_provider(EBNF)
-    trans_fac = lambda: partial(traverse, processing_table={"*": [flatten, remove_empty]})
+    trans_fac = lambda: partial(traverse, transformation_table={"*": [flatten, remove_empty]})
 
     def setup(self):
         self.save_dir = os.getcwd()
@@ -378,7 +383,9 @@ class TestLookahead:
                    for e in result.errors), str(result.errors)
         # Case 2: Lookahead string is not part of the test case; parser matches but for the mandatory continuation
         result = gr(self.cases['category']['match'][2], 'category')
-        assert any(e.code == MANDATORY_CONTINUATION_AT_EOF for e in result.errors)
+        # print(result.errors)
+        assert any(e.code in (MANDATORY_CONTINUATION_AT_EOF,
+                              MANDATORY_CONTINUATION_AT_EOF_NON_ROOT) for e in result.errors)
         errata = grammar_unit(self.cases, TestLookahead.grammar_fac, TestLookahead.trans_fac,
                               'REPORT_TestLookahead')
         assert not errata, str(errata)
@@ -387,6 +394,50 @@ class TestLookahead:
         assert errata
 
 
+void_tests = """
+[match:empty_line]
+M1: '''
+
+    '''
+M2: '''
+        # comment
+    '''
+"""
+
+
+class TestLookaheadDroppedTokens:
+    def setup(self):
+        self.save_dir = os.getcwd()
+        os.chdir(scriptpath)
+
+    def teardown(self):
+        clean_report('REPORT_void')
+        os.chdir(self.save_dir)
+
+    def test_lookahead_dropped_tokens(self):
+        void_grammar = '''@ whitespace   = horizontal
+        @ comment      = /#[^\\n]*/
+        document       = { empty_line } /\s*/ EOF
+        empty_line     = LF ~ &LF
+        LF             = /\\n/
+        EOF            = !/./ '''
+        void_parser_provider = grammar_provider(void_grammar)
+        void_transformer_provider = lambda : lambda _: _
+        void_test_unit = unit_from_config(void_tests, 'void_tests.ini')
+        errata = grammar_unit(void_test_unit, void_parser_provider, void_transformer_provider,
+                              'REPORT_void')
+        assert not errata
+        drop_clause = '''@ disposable   = EOF, LF, empty_line
+        @ drop         = whitespace, strings, EOF, LF, empty_line
+        '''
+        void_parser_provider = grammar_provider(drop_clause + void_grammar)
+        void_transformer_provider = lambda : lambda _: _
+        reset_unit(void_test_unit)
+        errata = grammar_unit(void_test_unit, void_parser_provider, void_transformer_provider,
+                              'REPORT_void')
+        assert not errata
+
+
 class TestSExpr:
     """
     Tests for S-expression handling.
@@ -415,10 +466,10 @@ class TestSExpr:
     def test_mock_syntax_tree_with_classes(self):
         sexpr = '(a:class1 (b:class2 x) (:class3 y) (c z))'
         tree = parse_sxpr(sexpr)
-        assert tree.tag_name == 'a'
-        assert tree.result[0].tag_name == 'b'
-        assert tree.result[1].tag_name == ':class3'
-        assert tree.result[2].tag_name == 'c'
+        assert tree.name == 'a'
+        assert tree.result[0].name == 'b'
+        assert tree.result[1].name == ':class3'
+        assert tree.result[2].name == 'c'
 
 
 if __name__ == "__main__":
